//
// Copyright (C) 2013-2016  Taiyou Sakashita, Hiroko Yamada
//
// Calculate Dailymean and Standard Deviation 2015/08/10
// Modified for using Julidan day 2016/07/01
//


import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Calendar;

import java.text.SimpleDateFormat;
import java.text.ParseException;

public class dailymean
{

// Length of the Data in Column/Line Direction
static int X;
static int Y;


// Year/Month/Day/Minute
static int YYYY;
static int MM;
static int DD;
static int MN;

// UTC Year/Month/Day/Hour
static int UYYYY;
static int UMM;
static int UDD;
static int UHH;

// Number of Used Files
static int nFile = 0;

// Number of Used Files for Standard Deviation
static int nFile_std = 0;

// Error Hour Threshold
static int Error_HH_THS = 7 + 3; // 11-17(7 hour is no data) plus 3 error hour )


// Present Error Hour
static int P_Error_HH;

// Dailymean
static double DM[][];

// Standard Deviation
static double STD[][];

// File Type
static String ftype;

// Directory Name to Read Original Data
static String input_Day_DIR;

// Directory Name to Write Dailymean,Monthlymean & STD Data
static String Day_Mon_MnSTD_DIR;

// TIME DIFF 
static double TIME_DIFF;
static String TIME_Code;

// Julian day local 
static double jd_local = 0;
static double jd_std_local = 0;
static int FLG_DAILY = 0;
static int FLG_STD = 1;



public static void main(String args[])

{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );

  int I;

  // Filename to Write Dailymean Data
  String wfilename;

  // Filename to Write Standard Deviation Data
  String std_filename;

  // Year/Month/Day/Minute
  YYYY = Integer.valueOf(args[0]).intValue();
  MM   = Integer.valueOf(args[1]).intValue();
  DD   = Integer.valueOf(args[2]).intValue();

  //output/YYYYMM Dir confirm and create
  File newfile = new File("output/" + YYYY + df.format(MM));
   	if (newfile.exists()) {
		System.out.println( newfile + " already exists");
	}else if (newfile.mkdir()){
    		System.out.println("Succeeded to create a directory");
  	}else{
    		System.out.println("Failed to create a directory");
  	}

  date_errorcheck();
  initialization();
  
  jday jday_jul = new jday();
 
  double jd_jul_12ji=(double)jday_jul.ymd2jd(YYYY,MM,DD);
  jd_local= jd_jul_12ji - (double)12/24-(double)TIME_DIFF/24;
  jd_std_local= jd_local;
 	
  // Calculate Dailymean
  cal_dailymean();
  String ftype_cut = ftype.substring(0, ftype.length()-4);
  wfilename = Day_Mon_MnSTD_DIR + YYYY + df.format(MM) + "/" + YYYY + df.format(MM) + df.format(DD) +"." + TIME_Code + ftype_cut + ".dm.bin";


  bin bb = new bin();

  // Write Dailymean Data
  if(nFile > 0)
  {
     bb.d2bin(wfilename, X, Y, DM);

     // Calculate Standard Deviation
     cal_std(input_Day_DIR);

	 std_filename = Day_Mon_MnSTD_DIR + YYYY + df.format(MM) + "/" +  YYYY + df.format(MM) + df.format(DD) +"." + TIME_Code + ftype_cut + ".std.bin";

     // Write Standard Deviation Data
     if(nFile_std > 0) bb.d2bin(std_filename, X, Y, STD);

  }//fi


  return;


}//main


//public static void cal_dailymean(String TDIR)
public static void cal_dailymean()
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  int I;

  // Filename of Total Data
  String tfilename;


  P_Error_HH = 0;


  // Calculate the Sum of Hourly Data
  for(I=0;I<=23;I++)
  {
     cal_hourlymean(I);

  }//I

  if(P_Error_HH >= Error_HH_THS)
  {
     System.exit(1);

  }//fi
//     System.out.println("nFile"+ nFile);


  // Calculate Dailymean
  if(nFile > 0) DM = cal_mean_d(DM, 24);


  return;


}//cal_dailymean


public static void cal_hourlymean(int PHH)
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );

  int I;

  // Number of Data of 1 Hour
  int nFile_HH;

  // UTC Time
  int UTime[] = new int[5+1];

  // Data Read
  float DR[][] = new float[Y+1][X+1];

  // Hourly Mean
  float HM[][] = new float[Y+1][X+1];

  // Filename of Original Data
  String filename;

  File file;

  UTime = shift_time(FLG_DAILY);

  UYYYY = UTime[1];
  UMM = UTime[2];
  UDD = UTime[3];
  UHH = UTime[4];

  bin bb = new bin();

  nFile_HH = 0;

  for(MN=0;MN<=59;MN++)
  {
     filename = input_Day_DIR + UYYYY + df.format(UMM) + "/" + YYYY + df.format(UMM) + df.format(UDD) + df.format(UHH) + df.format(MN) + ftype;

     file = new File(filename);

     if(file.exists())
     {

        DR = bb.bin2f(filename, X, Y);
        add_data_f(DR, HM);
        nFile_HH++;

     }//fi
  }//MN

  if(nFile_HH > 0)
  {
     HM = cal_mean_f(HM, nFile_HH);
     nFile++;
     add_data_d(HM, DM);

  }//fi


  if(nFile_HH == 0)
  {
     P_Error_HH++;
     System.out.println("Data of " + UYYYY + df.format(UMM) + df.format(UDD) + df.format(UHH) + " is not found");

  }//fi


  return;


}//cal_hourlymean


public static void cal_std(String input_Day_DIR)
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  int I,J;

  // Data Read
  float DR[][] = new float[Y+1][X+1];

  // Filename of Original Data
  String filename;

  // UTC Time
  int UTime[] = new int[5+1];

  File file;


  bin bb = new bin();

  // Calculate Variance
  for(I=0;I<=23;I++)
  {
     UTime = shift_time(FLG_STD);

     UYYYY = UTime[1];
     UMM = UTime[2];
     UDD = UTime[3];
     UHH = UTime[4];

     for(MN=0;MN<=59;MN++)
     {
        filename =input_Day_DIR + UYYYY + df.format(UMM) + "/" + UYYYY + df.format(UMM) + df.format(UDD) + df.format(UHH) + df.format(MN) + ftype;

        file = new File(filename);

        if(file.exists())
         {
           DR = bb.bin2f(filename, X, Y);
           cal_var(DR);
           nFile_std++;

        }//fi
     }//MN
  }//I

  // Calculate Standard Deviation
  if(nFile_std > 0) 
  {
     STD = cal_mean_d(STD, nFile_std);
     standard_deviation();

  }//fi


  return;


}//cal_std


public static void add_data_d(float DR[][], double ADD[][])
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        ADD[I][J] = ADD[I][J] + DR[I][J];

     }//J
  }//I


  return;


}//add_data_d


public static void add_data_f(float DR[][], float ADD[][])
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        ADD[I][J] = ADD[I][J] + DR[I][J];

     }//J
  }//I


  return;


}//add_data_f


public static float[][] cal_mean_f(float DD[][], int N)
{

  int I,J;

  // Result
  float RR[][] = new float[Y+1][X+1];


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[I][J] = DD[I][J] / N;

     }//J
  }//I


  return RR;


}//cal_mean_f


public static double[][] cal_mean_d(double DD[][], int N)
{

  int I,J;

  // Result
  double RR[][] = new double[Y+1][X+1];


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        RR[I][J] = DD[I][J] / N;

     }//J
  }//I


  return RR;


}//cal_mean_d


public static void cal_var(float DR[][])
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[I][J] = STD[I][J] + (DR[I][J] - DM[I][J]) * (DR[I][J] - DM[I][J]);

     }//J
  }//I


  return;


}//cal_var


public static void standard_deviation()
{

  int I,J;


  for(I=1;I<=Y;I++)
  {
     for(J=1;J<=X;J++)
     {
        STD[I][J] = Math.sqrt(STD[I][J]);

     }//J
  }//I


  return;


}//standard_deviation


public static int get_lastday(int Year, int Month)
{

  // Last Day of the Month
  int Last_DD;

  Calendar time = Calendar.getInstance();

  time.set(Calendar.YEAR, Year);
  time.set(Calendar.MONTH, Month-1);
  time.set(Calendar.DAY_OF_MONTH, 1);
  time.set(Calendar.HOUR_OF_DAY, 0);
  time.set(Calendar.MINUTE, 0);

  Last_DD = time.getActualMaximum(Calendar.DAY_OF_MONTH);


  return Last_DD;


}//get_lastday


public static void date_errorcheck()
{

  DecimalFormat df = new DecimalFormat();
  df.applyLocalizedPattern( "00" );


  // Last Day of the Month
  int Last_DD;


  // Year Check
  if(YYYY <= 0)
  {
     System.out.println("Year " + YYYY + " is less than or equal to 0");
     System.exit(1);

  }//fi

  // Month Check
  if(MM <= 0 || MM > 12)
  {
     System.out.println("Month " + MM + " does not exist");
     System.exit(1);

  }//fi

  // Day Check
  Last_DD = get_lastday(YYYY, MM);

  if(DD > Last_DD || DD <= 0)
  {
     System.out.println("Date of " + YYYY + df.format(MM) + df.format(DD) + " does not exist");
     System.exit(1);

  }//fi
  

  return;


}//date_errorcheck



public static int[] shift_time(int flg)
{

  	// Shifted Year/Month/Day/Hour/Minute
  	int Shift_Time[] = new int[5+1];
	double jd=0;
	 
	if(flg == FLG_DAILY){
		jd_local= jd_local+(double)1/24; // Julian local day
		jd = jd_local;
 	}
	else if(flg == FLG_STD){
		jd_std_local= jd_std_local+(double)1/24; // Julian local day
		jd = jd_std_local;
	}
	
	int ans[] = new int[7+1];

  	jday jday_jul2 = new jday();

	ans = jday_jul2.jd2ymdhn(jd);

	Shift_Time[1] = ans[1]; //YYYY
  	Shift_Time[2] = ans[2];	//MM
  	Shift_Time[3] = ans[3];	//DD
  	Shift_Time[4] = ans[4];	//HH
 	Shift_Time[5] = ans[5]; //MIN


  return Shift_Time;


}//shift_time



public static void initialization()
{

  read_formatfile();


  DM = new double[Y+1][X+1];
  STD = new double[Y+1][X+1];


  return;


}//initialization


static public void read_formatfile()
{

  // Present Line
  int pline=1;

  String filename;
  String readline;
  String param[] = new String [100+1]; 


  filename = "config/format.txt";


  // Read Format File
  try
  {
       
    System.out.print(filename +"  Loading ...");

    BufferedReader br = new BufferedReader(new FileReader(filename));

    while( (readline = br.readLine()) != null ){

        StringTokenizer st = new StringTokenizer(readline);

	 param[pline] =  st.nextToken(", "); // "," or " "で文節を区切り文字列を取得できる

        pline++;
                 
    }//while

    br.close();
	
  }catch (IOException e){

      System.err.println("Initialize error\n");
      System.exit(1);

   }//try

   System.out.println(" finish");


  TIME_DIFF = Double.valueOf(param[1]).doubleValue();
  TIME_Code = param[2];

  
  X = Integer.valueOf(param[3]).intValue();
  Y = Integer.valueOf(param[4]).intValue();

  ftype = param[5];

  input_Day_DIR = param[6];
  Day_Mon_MnSTD_DIR = param[7];

  return;


}//read_formatfile

}//EOF

